"""Tests pokemon/gen_2/_memory.py."""

import unittest

import numpy as np
from pyboy import PyBoy

from gymboy.environments.pokemon.gen_2._memory import (
    _badges,
    _exps,
    _game_area,
    _hps,
    _levels,
    _max_hps,
    _max_pps,
    _money,
    _mother_money,
    _moves,
    _own_money,
    _owned_pokemons,
    _pokemon_ids,
    _pps,
    _seen_pokemons,
    _team_size,
)


class TestMemory(unittest.TestCase):
    """Tests the methods under the pokemon/gen_2/_memory.py file."""

    def setUp(self):
        self.rom_path = "./resources/roms/pokemon/gen_2/pokemon_gold.gbc"
        self.init_state_path1 = (
            "./resources/states/pokemon/gen_2/pokemon_gold_after_intro.state"
        )
        self.init_state_path2 = (
            "./resources/states/pokemon/gen_2/pokemon_gold_after_first_pokemon.state"
        )
        self.init_state_path3 = (
            "./resources/states/pokemon/gen_2/pokemon_gold_after_second_order.state"
        )

        self.pyboy1 = PyBoy(self.rom_path, sound_emulated=False)
        with open(self.init_state_path1, "rb") as f:
            self.pyboy1.load_state(f)
        self.pyboy1.tick(1)

        self.pyboy2 = PyBoy(self.rom_path, sound_emulated=False)
        with open(self.init_state_path2, "rb") as f:
            self.pyboy2.load_state(f)
        self.pyboy2.tick(1)

        self.pyboy3 = PyBoy(self.rom_path, sound_emulated=False)
        with open(self.init_state_path3, "rb") as f:
            self.pyboy3.load_state(f)
        self.pyboy3.tick(1)

    def tearDown(self):
        self.pyboy1.stop()
        self.pyboy2.stop()
        self.pyboy3.stop()

    def test_badges(self):
        """Tests the badges() method."""
        self.assertEqual(0, _badges(self.pyboy1))
        self.assertEqual(0, _badges(self.pyboy2))
        self.assertEqual(2, _badges(self.pyboy3))

    def test_own_money(self):
        """Tests the own_money() method."""
        self.assertEqual(3000, _own_money(self.pyboy1))
        self.assertEqual(3000, _own_money(self.pyboy2))
        self.assertEqual(6380, _own_money(self.pyboy3))

    def test_mother_money(self):
        """Tests the mother_money() method."""
        self.assertEqual(0, _mother_money(self.pyboy1))
        self.assertEqual(0, _mother_money(self.pyboy2))
        self.assertEqual(1639, _mother_money(self.pyboy3))

    def test_money(self):
        """Tests the money() method."""
        self.assertEqual(3000, _money(self.pyboy1))
        self.assertEqual(3000, _money(self.pyboy2))
        self.assertEqual(8019, _money(self.pyboy3))

    def test_pokemon_ids(self):
        """Tests the pokemon_ids() method."""
        np.testing.assert_allclose([0, 0, 0, 0, 0, 0], _pokemon_ids(self.pyboy1))
        np.testing.assert_allclose([155, 0, 0, 0, 0, 0], _pokemon_ids(self.pyboy2))
        np.testing.assert_allclose(
            [74, 41, 187, 175, 179, 156], _pokemon_ids(self.pyboy3)
        )

    def test_team_size(self):
        """Tests the team_size() method."""
        self.assertEqual(0, _team_size(self.pyboy1))
        self.assertEqual(1, _team_size(self.pyboy2))
        self.assertEqual(6, _team_size(self.pyboy3))

    def test_levels(self):
        """Tests the levels() method."""
        np.testing.assert_allclose([0, 0, 0, 0, 0, 0], _levels(self.pyboy1))
        np.testing.assert_allclose([5, 0, 0, 0, 0, 0], _levels(self.pyboy2))
        np.testing.assert_allclose([18, 10, 12, 5, 6, 20], _levels(self.pyboy3))

    def test_hps(self):
        """Tests the hps() method."""
        np.testing.assert_allclose([0, 0, 0, 0, 0, 0], _hps(self.pyboy1))
        np.testing.assert_allclose([19, 0, 0, 0, 0, 0], _hps(self.pyboy2))
        np.testing.assert_allclose([46, 30, 34, 0, 24, 58], _hps(self.pyboy3))

    def test_max_hps(self):
        """Tests the max_hps() method."""
        np.testing.assert_allclose([0, 0, 0, 0, 0, 0], _max_hps(self.pyboy1))
        np.testing.assert_allclose([19, 0, 0, 0, 0, 0], _max_hps(self.pyboy2))
        np.testing.assert_allclose([46, 30, 34, 18, 24, 58], _max_hps(self.pyboy3))

    def test_exps(self):
        """Tests the exps() method."""
        np.testing.assert_allclose([0, 0, 0, 0, 0, 0], _exps(self.pyboy1))
        np.testing.assert_allclose([135, 0, 0, 0, 0, 0], _exps(self.pyboy2))
        np.testing.assert_allclose(
            [4573, 1013, 1155, 100, 179, 6103], _exps(self.pyboy3)
        )

    def test_moves(self):
        """Tests the moves() method."""
        np.testing.assert_allclose(
            [
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
            ],
            _moves(self.pyboy1),
        )
        np.testing.assert_allclose(
            [
                [33, 43, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
            ],
            _moves(self.pyboy2),
        )
        np.testing.assert_allclose(
            [
                [33, 111, 88, 222],
                [141, 48, 0, 0],
                [150, 235, 39, 33],
                [45, 204, 0, 0],
                [33, 45, 0, 0],
                [33, 43, 108, 52],
            ],
            _moves(self.pyboy3),
        )

    def test_pps(self):
        """Tests the pps() method."""
        np.testing.assert_allclose(
            [
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
            ],
            _pps(self.pyboy1),
        )
        np.testing.assert_allclose(
            [
                [35, 30, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
            ],
            _pps(self.pyboy2),
        )
        np.testing.assert_allclose(
            [
                [35, 40, 15, 30],
                [15, 20, 0, 0],
                [40, 5, 30, 35],
                [40, 20, 0, 0],
                [35, 40, 0, 0],
                [35, 30, 20, 25],
            ],
            _pps(self.pyboy3),
        )

    def test_max_pps(self):
        """Tests the max_pps() method."""
        np.testing.assert_allclose(
            [
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
            ],
            _max_pps(self.pyboy1),
        )
        np.testing.assert_allclose(
            [
                [35, 30, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
                [0, 0, 0, 0],
            ],
            _max_pps(self.pyboy2),
        )
        np.testing.assert_allclose(
            [
                [35, 40, 15, 30],
                [15, 20, 0, 0],
                [40, 5, 30, 35],
                [40, 20, 0, 0],
                [35, 40, 0, 0],
                [35, 30, 20, 25],
            ],
            _max_pps(self.pyboy3),
        )

    def test_seen_pokemons(self):
        """Tests the seen_pokemons() method."""
        self.assertEqual(0, _seen_pokemons(self.pyboy1))
        self.assertEqual(1, _seen_pokemons(self.pyboy2))
        self.assertEqual(31, _seen_pokemons(self.pyboy3))

    def test_owned_pokemons(self):
        """Tests the owned_pokemons() method."""
        self.assertEqual(0, _owned_pokemons(self.pyboy1))
        self.assertEqual(1, _owned_pokemons(self.pyboy2))
        self.assertEqual(6, _owned_pokemons(self.pyboy3))

    def test_game_area(self):
        """Tests the game_area() method."""
        np.testing.assert_allclose(
            _game_area(self.pyboy1),
            [
                [
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    258,
                    258,
                    258,
                    258,
                    258,
                    258,
                    258,
                    258,
                    258,
                    258,
                    261,
                    262,
                    324,
                    325,
                    320,
                    321,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    258,
                    258,
                    258,
                    258,
                    267,
                    268,
                    258,
                    258,
                    315,
                    316,
                    277,
                    278,
                    340,
                    341,
                    336,
                    337,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    257,
                    257,
                    257,
                    257,
                    283,
                    284,
                    322,
                    323,
                    331,
                    332,
                    293,
                    294,
                    257,
                    257,
                    257,
                    257,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    257,
                    257,
                    257,
                    257,
                    299,
                    300,
                    338,
                    339,
                    347,
                    348,
                    309,
                    310,
                    257,
                    257,
                    257,
                    257,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    257,
                    257,
                    257,
                    257,
                    304,
                    305,
                    305,
                    306,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    0,
                    1,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    2,
                    3,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    259,
                    260,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    272,
                    273,
                    273,
                    274,
                    257,
                    257,
                    257,
                    257,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    275,
                    276,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    288,
                    289,
                    289,
                    290,
                    257,
                    257,
                    257,
                    257,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    291,
                    292,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    304,
                    305,
                    305,
                    306,
                    257,
                    257,
                    257,
                    257,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    307,
                    308,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    257,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                ],
                [
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                    256,
                ],
            ],
        )
        np.testing.assert_allclose(
            _game_area(self.pyboy2),
            np.array(
                [
                    [
                        286,
                        287,
                        286,
                        287,
                        272,
                        273,
                        273,
                        273,
                        273,
                        273,
                        273,
                        273,
                        273,
                        273,
                        273,
                        274,
                        286,
                        287,
                        286,
                        287,
                    ],
                    [
                        302,
                        303,
                        302,
                        303,
                        269,
                        270,
                        270,
                        270,
                        270,
                        270,
                        270,
                        270,
                        270,
                        270,
                        270,
                        271,
                        302,
                        303,
                        302,
                        303,
                    ],
                    [
                        302,
                        303,
                        302,
                        303,
                        269,
                        270,
                        270,
                        270,
                        270,
                        270,
                        270,
                        270,
                        270,
                        270,
                        270,
                        271,
                        302,
                        303,
                        302,
                        303,
                    ],
                    [
                        318,
                        319,
                        21,
                        20,
                        266,
                        267,
                        267,
                        267,
                        267,
                        267,
                        267,
                        267,
                        267,
                        267,
                        267,
                        268,
                        318,
                        319,
                        318,
                        319,
                    ],
                    [
                        261,
                        261,
                        23,
                        22,
                        282,
                        283,
                        283,
                        283,
                        283,
                        283,
                        283,
                        283,
                        283,
                        283,
                        283,
                        284,
                        286,
                        287,
                        286,
                        287,
                    ],
                    [
                        261,
                        261,
                        261,
                        261,
                        282,
                        294,
                        294,
                        283,
                        283,
                        283,
                        283,
                        283,
                        283,
                        294,
                        294,
                        284,
                        302,
                        303,
                        302,
                        303,
                    ],
                    [
                        261,
                        261,
                        334,
                        335,
                        282,
                        283,
                        283,
                        283,
                        311,
                        312,
                        283,
                        283,
                        283,
                        283,
                        283,
                        284,
                        302,
                        303,
                        302,
                        303,
                    ],
                    [
                        261,
                        261,
                        350,
                        351,
                        257,
                        258,
                        258,
                        258,
                        0,
                        1,
                        258,
                        258,
                        258,
                        258,
                        258,
                        278,
                        318,
                        319,
                        318,
                        319,
                    ],
                    [
                        261,
                        261,
                        261,
                        261,
                        262,
                        262,
                        262,
                        262,
                        2,
                        3,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        261,
                        261,
                        261,
                        261,
                    ],
                    [
                        261,
                        261,
                        261,
                        261,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        261,
                        261,
                        261,
                        261,
                    ],
                    [
                        261,
                        261,
                        261,
                        261,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        261,
                        261,
                        334,
                        335,
                    ],
                    [
                        261,
                        261,
                        261,
                        261,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        261,
                        261,
                        350,
                        351,
                    ],
                    [
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                    ],
                    [
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                    ],
                    [
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                    ],
                    [
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        28,
                        29,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                    ],
                    [
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        30,
                        31,
                        262,
                        262,
                        334,
                        335,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                    ],
                    [
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        350,
                        351,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                    ],
                ]
            ),
        )
        np.testing.assert_allclose(
            _game_area(self.pyboy3),
            np.array(
                [
                    [
                        261,
                        261,
                        286,
                        287,
                        286,
                        287,
                        272,
                        273,
                        273,
                        273,
                        377,
                        378,
                        378,
                        378,
                        378,
                        378,
                        378,
                        378,
                        378,
                        379,
                    ],
                    [
                        261,
                        261,
                        275,
                        277,
                        275,
                        277,
                        269,
                        270,
                        270,
                        270,
                        380,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        261,
                        261,
                        275,
                        277,
                        275,
                        277,
                        269,
                        270,
                        270,
                        270,
                        380,
                        237,
                        143,
                        142,
                        138,
                        234,
                        131,
                        132,
                        151,
                        380,
                    ],
                    [
                        332,
                        332,
                        318,
                        319,
                        318,
                        319,
                        266,
                        267,
                        267,
                        267,
                        380,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        261,
                        261,
                        286,
                        287,
                        286,
                        287,
                        282,
                        263,
                        263,
                        263,
                        380,
                        383,
                        143,
                        142,
                        138,
                        234,
                        140,
                        142,
                        141,
                        380,
                    ],
                    [
                        261,
                        261,
                        275,
                        277,
                        275,
                        277,
                        282,
                        263,
                        263,
                        263,
                        380,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        261,
                        261,
                        275,
                        277,
                        275,
                        277,
                        282,
                        263,
                        311,
                        312,
                        380,
                        383,
                        143,
                        128,
                        130,
                        138,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        261,
                        261,
                        318,
                        319,
                        318,
                        319,
                        257,
                        258,
                        0,
                        1,
                        380,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        2,
                        3,
                        380,
                        383,
                        368,
                        369,
                        134,
                        132,
                        128,
                        145,
                        383,
                        380,
                    ],
                    [
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        380,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        380,
                        383,
                        134,
                        142,
                        139,
                        131,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        262,
                        380,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        273,
                        273,
                        273,
                        273,
                        273,
                        274,
                        261,
                        261,
                        261,
                        261,
                        380,
                        383,
                        146,
                        128,
                        149,
                        132,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        380,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        143,
                        142,
                        138,
                        234,
                        140,
                        142,
                        141,
                        383,
                        383,
                        383,
                        380,
                        383,
                        142,
                        143,
                        147,
                        136,
                        142,
                        141,
                        383,
                        380,
                    ],
                    [
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        380,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        163,
                        160,
                        179,
                        160,
                        161,
                        160,
                        178,
                        164,
                        383,
                        383,
                        380,
                        383,
                        132,
                        151,
                        136,
                        147,
                        383,
                        383,
                        383,
                        380,
                    ],
                    [
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        383,
                        381,
                        378,
                        378,
                        378,
                        378,
                        378,
                        378,
                        378,
                        378,
                        382,
                    ],
                ]
            ),
        )


if __name__ == "__main__":
    unittest.main()
